<?php

declare(strict_types=1);

namespace Drupal\ui_patterns_paragraphs\Plugin\UiPatterns\Source;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\Context\ContextDefinition;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\ui_patterns\Attribute\Source;
use Drupal\ui_patterns\SourcePluginBase;

/**
 * Plugin implementation of the source.
 */
#[Source(
  id: 'paragraphs_behaviors',
  label: new TranslatableMarkup('Paragraphs behaviors'),
  description: new TranslatableMarkup('Data from the paragraphs behaviors.'),
  prop_types: ['string'],
  context_definitions: [
    'entity' => new ContextDefinition('entity:paragraph', label: new TranslatableMarkup('Paragraph entity'), required: TRUE),
  ]
)]
class ParagraphsBehaviorsSource extends SourcePluginBase {

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state): array {
    /** @var \Drupal\paragraphs\Entity\Paragraph $paragraphs_entity */
    $paragraphs_entity = $this->getContextValue('entity');
    $paragraphs_type = $paragraphs_entity->getParagraphType();

    $options = [];
    foreach ($paragraphs_type->getEnabledBehaviorPlugins() as $plugin_id => $plugin) {
      $plugin_label = (string) $plugin->getPluginDefinition()['label'];
      $options[$plugin_label] = [];
      // @todo Handle multi level values.
      foreach (array_keys($plugin->defaultConfiguration()) as $key) {
        $options[$plugin_label][$plugin_id . ':' . $key] = $key;
      }
    }

    $form['behavior'] = [
      '#type' => 'select',
      '#title' => $this->t('Behavior'),
      '#required' => TRUE,
      '#options' => $options,
      '#default_value' => $this->getSetting('behavior'),
    ];

    $form['default_value'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Default value'),
      '#default_value' => $this->getSetting('default_value'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function getPropValue(): mixed {
    /** @var \Drupal\paragraphs\Entity\Paragraph $entity */
    $entity = $this->getContextValue('entity');

    [$plugin_id, $key] = explode(':', $this->settings['behavior']);
    $key = explode('.', $key);
    return $entity->getBehaviorSetting($plugin_id, $key, $this->settings['default_value'] ?? NULL);
  }

}
